@@ -307,7 +307,7 @@ ATIMach64SetDefaultAttributes
 }
 
 /*
- * There is a one-to-one correspondance between elements of the following array
+ * There is a one-to-one correspondence between elements of the following array
  * and those of ATIMach64Attribute.
  */
 static ATIMach64AttributeRec ATIMach64AttributeInfo[nATIMach64Attribute] =
@@ -524,7 +524,7 @@ ATIMach64RemoveLinearCallback
     FBLinearPtr pLinear
 )
 {
-    ATIPtr pATI = ATIPTR(xf86Screens[pLinear->pScreen->myNum]);
+    ATIPtr pATI = ATIPTR(xf86ScreenToScrn(pLinear->pScreen));
 
     pATI->pXVBuffer = NULL;
     outf(OVERLAY_SCALE_CNTL, SCALE_EN);
@@ -1349,7 +1349,7 @@ ATIMach64XVInitialiseAdaptor
     XF86VideoAdaptorPtr **pppAdaptor
 )
 {
-    ScreenPtr           pScreen    = screenInfo.screens[pScreenInfo->scrnIndex];
+    ScreenPtr           pScreen    = xf86ScrnToScreen(pScreenInfo);
     ATIPtr              pATI       = ATIPTR(pScreenInfo);
     XF86VideoAdaptorPtr *ppAdaptor = NULL;
     XF86VideoAdaptorPtr pAdaptor;
@@ -1467,8 +1467,8 @@ ATIMach64XVInitialiseAdaptor
     if (pppAdaptor)
         *pppAdaptor = ppAdaptor;
     else {
-        xfree(ppAdaptor[0]);
-        xfree(ppAdaptor);
+        free(ppAdaptor[0]);
+        free(ppAdaptor);
     }
 
     return 1;
@@ -1505,9 +1505,9 @@ ATIXVFreeAdaptorInfo
         return;
 
     while (nAdaptor > 0)
-        xfree(ppAdaptor[--nAdaptor]);
+        free(ppAdaptor[--nAdaptor]);
 
-    xfree(ppAdaptor);
+    free(ppAdaptor);
 }
 
 /*
@@ -1552,6 +1552,9 @@ ATICloseXVideo
     ATIPtr      pATI
 )
 {
+    if (!pATI->Block1Base)
+        return;
+
     ATIMach64StopVideo(pScreenInfo, pATI, TRUE);
 
     REGION_UNINIT(pScreen, &pATI->VideoClip);
